/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 *
 * Contributors:
 *    IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.wtp.releng.tools.component.codecoverage;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.eclipse.jdt.core.util.ClassFormatException;
import org.eclipse.wtp.releng.tools.component.ILocation;
import org.eclipse.wtp.releng.tools.component.ILocationVisitor;
import org.eclipse.wtp.releng.tools.component.api.API2ComponentAPI;
import org.eclipse.wtp.releng.tools.component.api.ClassAPI;
import org.eclipse.wtp.releng.tools.component.api.ComponentAPI;
import org.eclipse.wtp.releng.tools.component.api.ComponentXMLVisitor;
import org.eclipse.wtp.releng.tools.component.api.MethodAPI;
import org.eclipse.wtp.releng.tools.component.api.PackageAPI;
import org.eclipse.wtp.releng.tools.component.api.TestCoverage;
import org.eclipse.wtp.releng.tools.component.classes.Class2Reference;
import org.eclipse.wtp.releng.tools.component.classes.IClassVisitor;
import org.eclipse.wtp.releng.tools.component.classes.LibVisitor;
import org.eclipse.wtp.releng.tools.component.internal.FileLocation;
import org.eclipse.wtp.releng.tools.component.internal.Location;
import org.eclipse.wtp.releng.tools.component.model.ComponentXML;
import org.eclipse.wtp.releng.tools.component.use.ClassUse;
import org.eclipse.wtp.releng.tools.component.use.MethodUse;
import org.eclipse.wtp.releng.tools.component.use.Source;
import org.eclipse.wtp.releng.tools.component.util.CommandOptionParser;

public class CodeCoverageScanner implements IClassVisitor
{
  private String api;
  private String src;
  private String use;
  private String outputDir;
  private Collection includes;
  private Collection excludes;
  private boolean html;
  private String xsl;

  public String getOutputDir()
  {
    return outputDir;
  }

  public void setOutputDir(String outputDir)
  {
    this.outputDir = addTrailingSeperator(outputDir);
  }

  public String getApi()
  {
    return api;
  }

  public void setApi(String api)
  {
    this.api = api;
  }

  public String getSrc()
  {
    return src;
  }

  public void setSrc(String src)
  {
    this.src = src;
  }

  public String getUse()
  {
    return use;
  }

  public void setUse(String use)
  {
    this.use = use;
  }

  public Collection getIncludes()
  {
    return includes;
  }

  public void setIncludes(Collection includes)
  {
    this.includes = includes;
  }

  public Collection getExcludes()
  {
    return excludes;
  }

  public void setExcludes(Collection excludes)
  {
    this.excludes = excludes;
  }

  public boolean isHtml()
  {
    return html;
  }

  public void setHtml(boolean html)
  {
    this.html = html;
  }

  public String getXsl()
  {
    return xsl;
  }

  public void setXsl(String xsl)
  {
    this.xsl = xsl;
  }

  public int getCache()
  {
    return cacheSize;
  }

  public void setCache(int cache)
  {
    this.cacheSize = cache;
  }

  private List compXMLs;

  public void execute()
  {
    // Collection component.xml files
    ILocation apiLocation = Location.createLocation(new File(api));
    ComponentXMLVisitor compXMLVisitor = new ComponentXMLVisitor();
    apiLocation.accept(compXMLVisitor);
    compXMLs = new ArrayList(compXMLVisitor.getCompXMLs());
    
    // Generate api-info.xml
    API2ComponentAPI api2CompXML = new API2ComponentAPI();
    api2CompXML.setApi(api);
    api2CompXML.setSrc(src);
    api2CompXML.setOutputDir(outputDir);
    api2CompXML.setIncludes(includes);
    api2CompXML.setExcludes(excludes);
    api2CompXML.execute();

    // Visit all .class files that uses APIs
    ILocation srcLocation = Location.createLocation(new File(use));
    LibVisitor libVisitor = new LibVisitor();
    srcLocation.accept(libVisitor);
    libVisitor.setClassVisitor(this);
    srcLocation.accept(libVisitor);
    try
    {
      for (Iterator it = cachedCompAPIs.iterator(); it.hasNext();)
        ((ComponentAPI)it.next()).save();
    }
    catch (IOException e)
    {
      throw new RuntimeException(e);
    }
    if (isHtml())
    {
      genHTML();
    }
  }

  private void genHTML()
  {
    ILocation outputLoc = Location.createLocation(new File(outputDir));
    outputLoc.accept(new ILocationVisitor()
    {
      public boolean accept(ILocation location)
      {
        if (location.getName().endsWith("api-info.xml"))
        {
          try
          {
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer(new StreamSource(xsl != null && xsl.length() > 0 ? Location.createLocation(new File(xsl)).getInputStream() : ClassLoader.getSystemResourceAsStream("org/eclipse/wtp/releng/tools/component/xsl/component-violation.xsl")));
            transformer.transform(new StreamSource(location.getInputStream()), new StreamResult(new FileOutputStream(((FileLocation)location.createSibling("component-violation.html")).getFile())));
          }
          catch (Throwable e)
          {
            e.printStackTrace();
          }
        }
        return true;
      }
    });
  }

  private Class2Reference class2Ref = new Class2Reference();

  public boolean visit(String pluginId, ILocation classLoc)
  {
    System.out.println(classLoc.getName());
    try
    {
      Source use = class2Ref.visit(classLoc);
      for (Iterator it = use.getClassUses().iterator(); it.hasNext();)
      {
        ClassUse classUse = (ClassUse)it.next();
        String className = classUse.getName();
        int i = className.lastIndexOf('.');
        String packageName = (i != -1) ? className.substring(0, i) : "";
        String localName = (i != -1) ? className.substring(i + 1) : className;
        String id = getComponentId(packageName);
        if (id != null)
        {
          ComponentAPI compAPI = getComponentAPI(id);
          PackageAPI pkgAPI = compAPI.getPackageAPI(packageName);
          if (pkgAPI != null)
          {
            ClassAPI classAPI = pkgAPI.getClassAPI(localName);
            if (classAPI != null)
            {
              for (Iterator it2 = classUse.getMethodUses().iterator(); it2.hasNext();)
              {
                MethodUse methodUse = (MethodUse)it2.next();
                MethodAPI methodAPI = classAPI.getMethodAPI(methodUse.getName(), methodUse.getDescriptor());
                if (methodAPI != null)
                {
                  TestCoverage tc = new TestCoverage();
                  tc.addTest(use.getName());
                  methodAPI.setTestCoverage(tc);
                }
              }
              /*
              for (Iterator it2 = classUse.getFieldUses().iterator(); it2.hasNext();)
              {
                FieldAPI fieldAPI = classAPI.getFieldAPI(((FieldUse)it2.next()).getName());
                if (fieldAPI != null)
                {
                  // set code coverage for this field
                }
              }
              */
            }
          }
        }
      }
    }
    catch (IOException e)
    {
      throw new RuntimeException(e);
    }
    catch (ClassFormatException e)
    {
      throw new RuntimeException(e);
    }
    return true;
  }

  private String cachedPackageName;
  private String cachedCompId;

  private String getComponentId(String packageName)
  {
    if (cachedPackageName != null && cachedPackageName.equals(packageName))
    {
      return cachedCompId;
    }
    for (Iterator it = compXMLs.iterator(); it.hasNext();)
    {
      ComponentXML compXML = (ComponentXML)it.next();
      if (compXML.getPackage(packageName) != null)
      {
        cachedPackageName = packageName;
        cachedCompId = compXML.getName();
        return cachedCompId;
      }
    }
    return null;
  }

  private int cacheSize = 20;
  private List cachedIds = new ArrayList(cacheSize);
  private List cachedCompAPIs = new ArrayList(cacheSize);

  private ComponentAPI getComponentAPI(String id) throws IOException
  {
    int index = cachedIds.indexOf(id);
    if (index != -1)
    {
      ComponentAPI compAPI = (ComponentAPI)cachedCompAPIs.get(index);
      if (index != 0)
      {
        cachedIds.remove(index);
        cachedCompAPIs.remove(index);
        cachedIds.add(0, id);
        cachedCompAPIs.add(0, compAPI);
      }
      return compAPI;
    }
    StringBuffer sb = new StringBuffer(outputDir);
    sb.append(id);
    sb.append("/api-info.xml");
    File file = new File(sb.toString());
    ComponentAPI compAPI = new ComponentAPI();
    compAPI.setName(id);
    compAPI.setLocation(new FileLocation(file));
    if (file.exists())
    {
      compAPI.load();
    }
    if (cachedCompAPIs.size() == cacheSize)
    {
      cachedIds.remove(cacheSize - 1);
      ((ComponentAPI)cachedCompAPIs.remove(cacheSize - 1)).save();
    }
    cachedIds.add(0, id);
    cachedCompAPIs.add(0, compAPI);
    return compAPI;
  }

  protected String addTrailingSeperator(String s)
  {
    if (s != null && !s.endsWith("/") && !s.endsWith("\\"))
    {
      StringBuffer sb = new StringBuffer(s);
      sb.append('/');
      return sb.toString();
    }
    else
    {
      return s;
    }
  }

  public static void main(String[] args)
  {
    CommandOptionParser optionParser = new CommandOptionParser(args);
    Map options = optionParser.getOptions();
    Collection api = (Collection)options.get("api");
    Collection src = (Collection)options.get("src");
    Collection use = (Collection)options.get("use");
    Collection outputDir = (Collection)options.get("outputDir");
    Collection includes = (Collection)options.get("includes");
    Collection excludes = (Collection)options.get("excludes");
    Collection html = (Collection)options.get("html");
    Collection xsl = (Collection)options.get("xsl");
    Collection cache = (Collection)options.get("cache");
    if (src == null || api == null || outputDir == null || src.isEmpty() || api.isEmpty() || outputDir.isEmpty())
    {
      printUsage();
      System.exit(-1);
    }
    CodeCoverageScanner scanner = new CodeCoverageScanner();
    scanner.setApi((String)api.iterator().next());
    scanner.setSrc((String)src.iterator().next());
    scanner.setUse((String)use.iterator().next());
    scanner.setOutputDir((String)outputDir.iterator().next());
    scanner.setIncludes(includes);
    scanner.setExcludes(excludes);
    scanner.setHtml(html != null);
    scanner.setXsl(xsl != null && !xsl.isEmpty() ? (String)xsl.iterator().next() : null);
    if (cache != null && !cache.isEmpty())
    {
      try
      {
        scanner.setCache(Integer.parseInt((String)cache.iterator().next()));
      }
      catch (NumberFormatException e)
      {
      }
    }
    scanner.execute();
  }

  private static void printUsage()
  {
    System.out.println("Usage: java org.eclipse.wtp.releng.tools.component.codecoverage.CodeCoverage -api <api> -src <src> -use <use> -outputDir <outputDir> [-options]");
    System.out.println("");
    System.out.println("\t-api\t\t<api>\t\tlocation of your component.xml");
    System.out.println("\t-src\t\t<src>\t\tlocation of a Eclipse-based product (requires SDK build)");
    System.out.println("\t-use\t\t<use>\t\tlocation of the code that uses the APIs");
    System.out.println("\t-outputDir\t<outputDir>\toutput directory of component.xml files");
    System.out.println("");
    System.out.println("where options include:");
    System.out.println("");
    System.out.println("\t-includes\t<includes>\tspace seperated packages to include");
    System.out.println("\t-excludes\t<excludes>\tspace seperated packages to exclude");
    System.out.println("\t-html\t\t\t\tgenerate HTML results");
    System.out.println("\t-xsl\t<xsl>\t\tuse your own stylesheet. You must specify the -html option");
    System.out.println("\t-cache\t<cache>\t\tthis option controls the size of the api-info.xml cache (default size is 20)");
    System.out.println("\t\t\t\ta larger cache usually makes the scanner to run faster, but at the expense of using more memory");
  }
}